@@ -10,23 +10,25 @@ from alembic.autogenerate.render import _drop_table
 from alembic.ddl.base import RenameTable
 from alembic.ddl.base import format_table_name
 from alembic.ddl.base import visit_rename_table
+from alembic.ddl.sqlite import SQLiteImpl
 from alembic.operations import BatchOperations
 from alembic.operations import Operations
 from alembic.operations import ops
+from sqlalchemy import Column
 from sqlalchemy import text
+from sqlalchemy.dialects.mysql.base import MySQLDialect
 from sqlalchemy.dialects.sqlite.base import SQLiteDialect
 from sqlalchemy.ext.compiler import compiles
 from sqlalchemy.schema import DropTable
 from sqlalchemy.sql import func
 from sqlalchemy.types import TypeDecorator
 
-from geoalchemy2 import Column
 from geoalchemy2 import Geography
 from geoalchemy2 import Geometry
 from geoalchemy2 import Raster
-from geoalchemy2.dialects.common import _check_spatial_type
-from geoalchemy2.dialects.common import _get_gis_cols
-from geoalchemy2.dialects.common import _spatial_idx_name
+from geoalchemy2.admin.dialects.common import _check_spatial_type
+from geoalchemy2.admin.dialects.common import _get_gis_cols
+from geoalchemy2.admin.dialects.common import _spatial_idx_name
 
 writer = rewriter.Rewriter()
 """Rewriter object for Alembic."""
@@ -34,6 +36,12 @@ writer = rewriter.Rewriter()
 _SPATIAL_TABLES = set()
 
 
+class GeoPackageImpl(SQLiteImpl):
+    """Class to copy the Alembic implementation from SQLite to GeoPackage."""
+
+    __dialect__ = "geopackage"
+
+
 def _monkey_patch_get_indexes_for_sqlite():
     """Monkey patch SQLAlchemy to fix spatial index reflection."""
     normal_behavior = SQLiteDialect.get_indexes
@@ -41,10 +49,14 @@ def _monkey_patch_get_indexes_for_sqlite():
     def spatial_behavior(self, connection, table_name, schema=None, **kw):
         indexes = self._get_indexes_normal_behavior(connection, table_name, schema=None, **kw)
 
+        is_gpkg = connection.dialect.name == "geopackage"
+
         try:
             # Check that SpatiaLite was loaded into the DB
             is_spatial_db = connection.exec_driver_sql(
-                """PRAGMA main.table_info(geometry_columns)"""
+                """PRAGMA main.table_info({})""".format(
+                    "gpkg_geometry_columns" if is_gpkg else "geometry_columns"
+                )
             ).fetchall()
             if not is_spatial_db:
                 return indexes
@@ -52,15 +64,35 @@ def _monkey_patch_get_indexes_for_sqlite():
             return indexes
 
         # Get spatial indexes
-        spatial_index_query = text(
-            """SELECT *
-            FROM geometry_columns
-            WHERE f_table_name = '{}'
-            ORDER BY f_table_name, f_geometry_column;""".format(
-                table_name
+        if is_gpkg:
+            spatial_index_query = text(
+                """SELECT A.table_name, A.column_name, IFNULL(B.has_index, 0) AS has_index
+                FROM "gpkg_geometry_columns"
+                AS A
+                LEFT JOIN (
+                    SELECT table_name, column_name, COUNT(*) AS has_index
+                    FROM gpkg_extensions
+                    WHERE LOWER(table_name) = LOWER('{table_name}')
+                        AND extension_name = 'gpkg_rtree_index'
+                ) AS B
+                ON LOWER(A.table_name) = LOWER(B.table_name)
+                WHERE LOWER(A.table_name) = LOWER('{table_name}');
+            """.format(
+                    table_name=table_name
+                )
             )
-        )
+        else:
+            spatial_index_query = text(
+                """SELECT *
+                FROM geometry_columns
+                WHERE f_table_name = '{}'
+                ORDER BY f_table_name, f_geometry_column;""".format(
+                    table_name
+                )
+            )
+
         spatial_indexes = connection.execute(spatial_index_query).fetchall()
+
         if spatial_indexes:
             reflected_names = set([i["name"] for i in indexes])
             for idx in spatial_indexes:
@@ -88,6 +120,51 @@ def _monkey_patch_get_indexes_for_sqlite():
 _monkey_patch_get_indexes_for_sqlite()
 
 
+def _monkey_patch_get_indexes_for_mysql():
+    """Monkey patch SQLAlchemy to fix spatial index reflection."""
+    normal_behavior = MySQLDialect.get_indexes
+
+    def spatial_behavior(self, connection, table_name, schema=None, **kw):
+        indexes = self._get_indexes_normal_behavior(connection, table_name, schema=None, **kw)
+
+        # Get spatial indexes
+        has_index_query = """SELECT DISTINCT
+                COLUMN_NAME
+            FROM INFORMATION_SCHEMA.STATISTICS
+            WHERE TABLE_NAME = '{}' AND INDEX_TYPE = 'SPATIAL'""".format(
+            table_name
+        )
+        if schema is not None:
+            has_index_query += """ AND TABLE_SCHEMA = '{}'""".format(schema)
+        spatial_indexes = connection.execute(text(has_index_query)).fetchall()
+
+        if spatial_indexes:
+            reflected_names = set([i["name"] for i in indexes])
+            for idx in spatial_indexes:
+                idx_col = idx[0]
+                idx_name = _spatial_idx_name(table_name, idx_col)
+                if idx_name in reflected_names:
+                    continue
+                indexes.append(
+                    {
+                        "name": idx_name,
+                        "column_names": [idx_col],
+                        "unique": 0,
+                        "dialect_options": {"_column_flag": True},
+                    }
+                )
+                reflected_names.add(idx_name)
+
+        return indexes
+
+    spatial_behavior.__doc__ = normal_behavior.__doc__
+    MySQLDialect.get_indexes = spatial_behavior
+    MySQLDialect._get_indexes_normal_behavior = normal_behavior
+
+
+_monkey_patch_get_indexes_for_mysql()
+
+
 def render_item(obj_type, obj, autogen_context):
     """Add proper imports for spatial types."""
     if obj_type == "type" and isinstance(obj, (Geometry, Geography, Raster)):
@@ -115,6 +192,8 @@ def include_object(obj, name, obj_type, reflected, compare_to):
         or name.startswith("views_geometry_columns")
         or name.startswith("virts_geometry_columns")
         or name.startswith("idx_")
+        or name.startswith("gpkg_")
+        or name.startswith("vgpkg_")
     ):
         return False
     return True
@@ -446,7 +525,7 @@ def render_drop_geo_table(autogen_context, op):
 def create_geo_table(context, revision, op):
     """Replace the default CreateTableOp by a geospatial-specific one."""
     dialect = context.bind.dialect
-    gis_cols = _get_gis_cols(op, (Geometry, Geography, Raster), dialect, check_col_management=False)
+    gis_cols = _get_gis_cols(op, (Geometry, Geography, Raster), dialect)
 
     if gis_cols:
         new_op = CreateGeospatialTableOp(
@@ -467,9 +546,7 @@ def drop_geo_table(context, revision, op):
     """Replace the default DropTableOp by a geospatial-specific one."""
     dialect = context.bind.dialect
     table = op.to_table()
-    gis_cols = _get_gis_cols(
-        table, (Geometry, Geography, Raster), dialect, check_col_management=False
-    )
+    gis_cols = _get_gis_cols(table, (Geometry, Geography, Raster), dialect)
 
     if gis_cols:
         new_op = DropGeospatialTableOp(op.table_name, schema=op.schema)
