@@ -67,6 +67,8 @@ Reference
 
 """
 import re
+from typing import List
+from typing import Type
 
 from sqlalchemy import inspect
 from sqlalchemy.ext.compiler import compiles
@@ -74,14 +76,17 @@ from sqlalchemy.sql import annotation
 from sqlalchemy.sql import functions
 from sqlalchemy.sql.elements import ColumnElement
 
-from . import elements
-from ._functions import _FUNCTIONS
+from geoalchemy2 import elements
+from geoalchemy2._functions import _FUNCTIONS
+from geoalchemy2._functions_helpers import _get_docstring
 
+_GeoFunctionBase: Type[functions.GenericFunction]
+_GeoFunctionParent: Type[functions.GenericFunction]
 try:
     # SQLAlchemy < 2
 
-    from sqlalchemy.sql.functions import _GenericMeta
-    from sqlalchemy.util import with_metaclass
+    from sqlalchemy.sql.functions import _GenericMeta  # type: ignore
+    from sqlalchemy.util import with_metaclass  # type: ignore
 
     class _GeoGenericMeta(_GenericMeta):
         """Extend the registering mechanism of sqlalchemy.
@@ -91,7 +96,7 @@ try:
 
         _register = False
 
-        def __init__(cls, clsname, bases, clsdict):
+        def __init__(cls, clsname, bases, clsdict) -> None:
             # Register the function
             elements.function_registry.add(clsname.lower())
 
@@ -109,7 +114,7 @@ except ImportError:
             super().__init_subclass__()
 
         @classmethod
-        def _register_geo_function(cls, clsname, clsdict):
+        def _register_geo_function(cls, clsname, clsdict) -> None:
             # Check _register attribute status
             cls._register = getattr(cls, "_register", True)
 
@@ -125,52 +130,52 @@ except ImportError:
 
 
 class TableRowElement(ColumnElement):
-    inherit_cache = False
+    inherit_cache: bool = False
     """The cache is disabled for this class."""
 
-    def __init__(self, selectable):
+    def __init__(self, selectable: bool) -> None:
         self.selectable = selectable
 
     @property
-    def _from_objects(self):
+    def _from_objects(self) -> List[bool]:
         return [self.selectable]
 
 
-class ST_AsGeoJSON(_GeoFunctionBase):
+class ST_AsGeoJSON(_GeoFunctionBase):  # type: ignore
     """Special process for the ST_AsGeoJSON() function.
 
     This is to be able to work with its feature version introduced in PostGIS 3.
     """
 
-    name = "ST_AsGeoJSON"
-    inherit_cache = True
+    name: str = "ST_AsGeoJSON"
+    inherit_cache: bool = True
     """The cache is enabled for this class."""
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         expr = kwargs.pop("expr", None)
-        args = list(args)
+        args_list = list(args)
         if expr is not None:
-            args = [expr] + args
-        for idx, element in enumerate(args):
+            args_list = [expr] + args_list
+        for idx, element in enumerate(args_list):
             if isinstance(element, functions.Function):
                 continue
-            elif isinstance(element, elements.HasFunction):
+            elif isinstance(element, elements._SpatialElement):
                 if element.extended:
                     func_name = element.geom_from_extended_version
                     func_args = [element.data]
                 else:
                     func_name = element.geom_from
                     func_args = [element.data, element.srid]
-                args[idx] = getattr(functions.func, func_name)(*func_args)
+                args_list[idx] = getattr(functions.func, func_name)(*func_args)
             else:
                 try:
                     insp = inspect(element)
                     if hasattr(insp, "selectable"):
-                        args[idx] = TableRowElement(insp.selectable)
+                        args_list[idx] = TableRowElement(insp.selectable)
                 except Exception:
                     continue
 
-        _GeoFunctionParent.__init__(self, *args, **kwargs)
+        _GeoFunctionParent.__init__(self, *args_list, **kwargs)
 
     __doc__ = (
         'Return the geometry as a GeoJSON "geometry" object, or the row as a '
@@ -204,7 +209,7 @@ def _compile_table_row_thing(element, compiler, **kw):
     return compiled.split(".")[0]
 
 
-class GenericFunction(_GeoFunctionBase):
+class GenericFunction(_GeoFunctionBase):  # type: ignore
     """The base class for GeoAlchemy functions.
 
     This class inherits from ``sqlalchemy.sql.functions.GenericFunction``, so
@@ -235,45 +240,48 @@ class GenericFunction(_GeoFunctionBase):
     # sqlalchemy.sql.functions._registry. Only its children will be registered.
     _register = False
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         expr = kwargs.pop("expr", None)
-        args = list(args)
+        args_list = list(args)
         if expr is not None:
-            args = [expr] + args
-        for idx, elem in enumerate(args):
-            if isinstance(elem, elements.HasFunction):
+            args_list = [expr] + args_list
+        for idx, elem in enumerate(args_list):
+            if isinstance(elem, elements._SpatialElement):
                 if elem.extended:
                     func_name = elem.geom_from_extended_version
                     func_args = [elem.data]
                 else:
                     func_name = elem.geom_from
                     func_args = [elem.data, elem.srid]
-                args[idx] = getattr(functions.func, func_name)(*func_args)
-        _GeoFunctionParent.__init__(self, *args, **kwargs)
+                args_list[idx] = getattr(functions.func, func_name)(*func_args)
+        _GeoFunctionParent.__init__(self, *args_list, **kwargs)
 
 
-# Iterate through _FUNCTIONS and create GenericFunction classes dynamically
-for name, type_, doc in _FUNCTIONS:
-    attributes = {
-        "name": name,
-        "inherit_cache": True,
-    }
-    docs = []
+__all__ = [
+    "GenericFunction",
+    "ST_AsGeoJSON",
+    "TableRowElement",
+]
 
-    if isinstance(doc, tuple):
-        docs.append(doc[0])
-        docs.append("see http://postgis.net/docs/{0}.html".format(doc[1]))
-    elif doc is not None:
-        docs.append(doc)
-        docs.append("see http://postgis.net/docs/{0}.html".format(name))
 
-    if type_ is not None:
-        attributes["type"] = type_
+def _create_dynamic_functions() -> None:
+    # Iterate through _FUNCTIONS and create GenericFunction classes dynamically
+    for name, type_, doc in _FUNCTIONS:
+        attributes = {
+            "name": name,
+            "inherit_cache": True,
+            "__doc__": _get_docstring(name, doc, type_),
+        }
 
-        type_str = "{0}.{1}".format(type_.__module__, type_.__name__)
-        docs.append("Return type: :class:`{0}`.".format(type_str))
+        if type_ is not None:
+            attributes["type"] = type_
 
-    if len(docs) != 0:
-        attributes["__doc__"] = "\n\n".join(docs)
+        globals()[name] = type(name, (GenericFunction,), attributes)
+        __all__.append(name)
 
-    globals()[name] = type(name, (GenericFunction,), attributes)
+
+_create_dynamic_functions()
+
+
+def __dir__() -> List[str]:
+    return __all__
