@@ -31,12 +31,14 @@ from sqlalchemy.exc import SAWarning
 from sqlalchemy.sql import func
 from sqlalchemy.testing.assertions import ComparesTables
 
-import geoalchemy2
-import geoalchemy2.dialects
+import geoalchemy2.admin.dialects
 from geoalchemy2 import Geography
 from geoalchemy2 import Geometry
 from geoalchemy2 import Raster
-from geoalchemy2.dialects.sqlite import _get_spatialite_attrs
+from geoalchemy2.admin.dialects.geopackage import (
+    _get_spatialite_attrs as _get_spatialite_attrs_gpkg,
+)
+from geoalchemy2.admin.dialects.sqlite import _get_spatialite_attrs as _get_spatialite_attrs_sqlite
 from geoalchemy2.elements import WKBElement
 from geoalchemy2.elements import WKTElement
 from geoalchemy2.shape import from_shape
@@ -44,18 +46,18 @@ from geoalchemy2.shape import to_shape
 
 from . import check_indexes
 from . import format_wkt
-from . import get_postgis_version
+from . import get_postgis_major_version
 from . import select
 from . import skip_case_insensitivity
 from . import skip_pg12_sa1217
 from . import skip_postgis1
-from . import skip_postgis3
 from . import test_only_with_dialects
 
 SQLA_LT_2 = parse_version(SA_VERSION) <= parse_version("1.999")
 
 
 class TestAdmin:
+    @test_only_with_dialects("postgresql", "sqlite-spatialite3", "sqlite-spatialite4")
     def test_create_drop_tables(
         self,
         conn,
@@ -74,6 +76,91 @@ class TestAdmin:
         metadata.create_all(conn)
         metadata.drop_all(conn, checkfirst=True)
 
+    @test_only_with_dialects("postgresql", "mysql", "sqlite-spatialite3", "sqlite-spatialite4")
+    def test_nullable(self, conn, metadata, setup_tables, dialect_name):
+        # Define the table
+        t = Table(
+            "nullable_geom_type",
+            metadata,
+            Column("id", Integer, primary_key=True),
+            Column(
+                "geom_not_nullable",
+                Geometry(geometry_type=None, srid=4326, spatial_index=False, nullable=False),
+            ),
+            Column(
+                "geom_nullable",
+                Geometry(geometry_type=None, srid=4326, spatial_index=False, nullable=True),
+            ),
+            Column(
+                "geom_col_not_nullable",
+                Geometry(geometry_type=None, srid=4326, spatial_index=False),
+                nullable=False,
+            ),
+            Column(
+                "geom_col_nullable",
+                Geometry(geometry_type=None, srid=4326, spatial_index=False),
+                nullable=True,
+            ),
+        )
+
+        # Create the table
+        t.create(bind=conn)
+
+        conn.execute(
+            t.insert(),
+            [
+                {
+                    "geom_not_nullable": "SRID=4326;LINESTRING(0 0,1 1)",
+                    "geom_nullable": "SRID=4326;LINESTRING(0 0,1 1)",
+                    "geom_col_not_nullable": "SRID=4326;LINESTRING(0 0,1 1)",
+                    "geom_col_nullable": "SRID=4326;LINESTRING(0 0,1 1)",
+                },
+                {
+                    "geom_not_nullable": "SRID=4326;LINESTRING(0 0,1 1)",
+                    "geom_nullable": None,
+                    "geom_col_not_nullable": "SRID=4326;LINESTRING(0 0,1 1)",
+                    "geom_col_nullable": None,
+                },
+            ],
+        )
+
+        with pytest.raises((IntegrityError, OperationalError)):
+            with conn.begin_nested():
+                conn.execute(
+                    t.insert(),
+                    [
+                        {
+                            "geom_not_nullable": None,
+                            "geom_nullable": None,
+                            "geom_col_not_nullable": "SRID=4326;LINESTRING(0 0,1 1)",
+                            "geom_col_nullable": None,
+                        },
+                    ],
+                )
+
+        with pytest.raises((IntegrityError, OperationalError)):
+            with conn.begin_nested():
+                conn.execute(
+                    t.insert(),
+                    [
+                        {
+                            "geom_not_nullable": "SRID=4326;LINESTRING(0 0,1 1)",
+                            "geom_nullable": None,
+                            "geom_col_not_nullable": None,
+                            "geom_col_nullable": None,
+                        },
+                    ],
+                )
+
+        results = conn.execute(t.select())
+        rows = results.fetchall()
+
+        assert len(rows) == 2
+
+        # Drop the table
+        t.drop(bind=conn)
+
+    @test_only_with_dialects("postgresql", "mysql")
     def test_no_geom_type(self, conn):
         with pytest.warns(UserWarning, match="srid not enforced when geometry_type is None"):
             # Define the table
@@ -131,10 +218,10 @@ class TestAdmin:
             marks.append("after_drop")
             return
 
-        monkeypatch.setattr(geoalchemy2.dialects.common, "before_create", value=before_create)
-        monkeypatch.setattr(geoalchemy2.dialects.common, "after_create", value=after_create)
-        monkeypatch.setattr(geoalchemy2.dialects.common, "before_drop", value=before_drop)
-        monkeypatch.setattr(geoalchemy2.dialects.common, "after_drop", value=after_drop)
+        monkeypatch.setattr(geoalchemy2.admin.dialects.common, "before_create", value=before_create)
+        monkeypatch.setattr(geoalchemy2.admin.dialects.common, "after_create", value=after_create)
+        monkeypatch.setattr(geoalchemy2.admin.dialects.common, "before_drop", value=before_drop)
+        monkeypatch.setattr(geoalchemy2.admin.dialects.common, "after_drop", value=after_drop)
 
         metadata.drop_all(conn, checkfirst=True)
         metadata.create_all(conn)
@@ -143,16 +230,6 @@ class TestAdmin:
         assert marks == ["before_create", "after_create", "before_drop", "after_drop"]
 
 
-class TestMiscellaneous:
-    @test_only_with_dialects("sqlite")
-    def test_load_spatialite(self, monkeypatch, conn):
-        geoalchemy2.load_spatialite(conn.connection.dbapi_connection, None)
-
-        monkeypatch.delenv("SPATIALITE_LIBRARY_PATH")
-        with pytest.raises(RuntimeError):
-            geoalchemy2.load_spatialite(conn.connection.dbapi_connection, None)
-
-
 class TestInsertionCore:
     def test_insert(self, conn, Lake, setup_tables):
         # Issue inserts using DBAPI's executemany() method. This tests the
@@ -198,6 +275,7 @@ class TestInsertionCore:
         srid = conn.execute(row[1].ST_SRID()).scalar()
         assert srid == 4326
 
+    @test_only_with_dialects("postgresql", "sqlite")
     def test_insert_geom_poi(self, conn, Poi, setup_tables):
         conn.execute(
             Poi.__table__.insert(),
@@ -260,13 +338,16 @@ class TestSelectBindParam:
         srid = conn.execute(row[1].ST_SRID()).scalar()
         assert srid == 4326
 
-    def test_select_bindparam_WKBElement_extented(self, conn, Lake, setup_one_lake):
+    def test_select_bindparam_WKBElement_extented(self, conn, Lake, setup_one_lake, dialect_name):
         s = Lake.__table__.select()
         results = conn.execute(s)
         rows = results.fetchall()
         geom = rows[0][1]
         assert isinstance(geom, WKBElement)
-        assert geom.extended is True
+        if dialect_name == "mysql":
+            assert geom.extended is False
+        else:
+            assert geom.extended is True
 
         s = Lake.__table__.select().where(Lake.__table__.c.geom == bindparam("geom"))
         params = {"geom": geom}
@@ -285,7 +366,7 @@ class TestSelectBindParam:
 
 
 class TestInsertionORM:
-    def test_WKT(self, session, Lake, setup_tables):
+    def test_WKT(self, session, Lake, setup_tables, dialect_name, postgis_version):
         # With PostGIS 1.5:
         # IntegrityError: (IntegrityError) new row for relation "lake" violates
         # check constraint "enforce_srid_geom"
@@ -296,50 +377,74 @@ class TestInsertionORM:
         #
         # With PostGIS 3.0:
         # The SRID is taken from the Column definition so no error is reported
-        skip_postgis3(session.connection())
-
         lake = Lake("LINESTRING(0 0,1 1)")
         session.add(lake)
 
-        with pytest.raises((DataError, IntegrityError)):
+        if (dialect_name == "postgresql" and postgis_version < 3) or dialect_name == "sqlite":
+            with pytest.raises((DataError, IntegrityError)):
+                session.flush()
+        else:
             session.flush()
 
-    def test_WKTElement(self, session, Lake, setup_tables):
+    def test_WKTElement(self, session, Lake, setup_tables, dialect_name):
         lake = Lake(WKTElement("LINESTRING(0 0,1 1)", srid=4326))
         session.add(lake)
         session.flush()
         session.expire(lake)
         assert isinstance(lake.geom, WKBElement)
-        assert str(lake.geom) == (
-            "0102000020e6100000020000000000000000000000000000000000000000000"
-            "0000000f03f000000000000f03f"
-        )
+        if dialect_name == "mysql":
+            # Not extended case
+            assert str(lake.geom) == (
+                "0102000000020000000000000000000000000000000000000000000"
+                "0000000f03f000000000000f03f"
+            )
+        else:
+            assert str(lake.geom) == (
+                "0102000020e6100000020000000000000000000000000000000000000000000"
+                "0000000f03f000000000000f03f"
+            )
         wkt = session.execute(lake.geom.ST_AsText()).scalar()
         assert format_wkt(wkt) == "LINESTRING(0 0,1 1)"
         srid = session.execute(lake.geom.ST_SRID()).scalar()
         assert srid == 4326
 
-    def test_WKBElement(self, session, Lake, setup_tables):
+    def test_WKBElement(self, session, Lake, setup_tables, dialect_name):
         shape = LineString([[0, 0], [1, 1]])
         lake = Lake(from_shape(shape, srid=4326))
         session.add(lake)
         session.flush()
         session.expire(lake)
         assert isinstance(lake.geom, WKBElement)
-        assert str(lake.geom) == (
-            "0102000020e6100000020000000000000000000000000000000000000000000"
-            "0000000f03f000000000000f03f"
-        )
+        if dialect_name == "mysql":
+            # Not extended case
+            assert str(lake.geom) == (
+                "0102000000020000000000000000000000000000000000000000000"
+                "0000000f03f000000000000f03f"
+            )
+        else:
+            assert str(lake.geom) == (
+                "0102000020e6100000020000000000000000000000000000000000000000000"
+                "0000000f03f000000000000f03f"
+            )
         wkt = session.execute(lake.geom.ST_AsText()).scalar()
         assert format_wkt(wkt) == "LINESTRING(0 0,1 1)"
         srid = session.execute(lake.geom.ST_SRID()).scalar()
         assert srid == 4326
 
+    @test_only_with_dialects("postgresql", "mysql", "sqlite-spatialite3", "sqlite-spatialite4")
     def test_transform(self, session, LocalPoint, setup_tables):
+        if session.bind.dialect.name == "mysql":
+            # Explicitly skip MySQL dialect to show that there is an issue
+            pytest.skip(
+                reason=(
+                    "The SRID is not properly retrieved so an exception is raised. TODO: This "
+                    "should be fixed later"
+                )
+            )
         # Create new point instance
         p = LocalPoint()
-        p.geom = "SRID=4326;POINT(5 45)"  # Insert 2D geometry into 3D column
-        p.managed_geom = "SRID=4326;POINT(5 45)"  # Insert 2D geometry into 3D column
+        p.geom = "SRID=4326;POINT(5 45)"  # Insert geometry with wrong SRID
+        p.managed_geom = "SRID=4326;POINT(5 45)"  # Insert geometry with wrong SRID
 
         # Insert point
         session.add(p)
@@ -374,7 +479,7 @@ class TestInsertionORM:
 
 
 class TestUpdateORM:
-    def test_WKTElement(self, session, Lake, setup_tables):
+    def test_WKTElement(self, session, Lake, setup_tables, dialect_name):
         raw_wkt = "LINESTRING(0 0,1 1)"
         lake = Lake(WKTElement(raw_wkt, srid=4326))
         session.add(lake)
@@ -389,16 +494,17 @@ class TestUpdateORM:
         srid = session.execute(lake.geom.ST_SRID()).scalar()
         assert srid == 4326
 
-        # Set geometry to None
-        lake.geom = None
+        if dialect_name != "mysql":
+            # Set geometry to None
+            lake.geom = None
 
-        # Update in DB
-        session.flush()
+            # Update in DB
+            session.flush()
 
-        # Check what was updated in DB
-        assert lake.geom is None
-        cols = [Lake.id, Lake.geom]
-        assert session.execute(select(cols)).fetchall() == [(1, None)]
+            # Check what was updated in DB
+            assert lake.geom is None
+            cols = [Lake.id, Lake.geom]
+            assert session.execute(select(cols)).fetchall() == [(1, None)]
 
         # Reset geometry to initial value
         lake.geom = WKTElement(raw_wkt, srid=4326)
@@ -413,7 +519,7 @@ class TestUpdateORM:
         srid = session.execute(lake.geom.ST_SRID()).scalar()
         assert srid == 4326
 
-    def test_WKBElement(self, session, Lake, setup_tables):
+    def test_WKBElement(self, session, Lake, setup_tables, dialect_name):
         shape = LineString([[0, 0], [1, 1]])
         initial_value = from_shape(shape, srid=4326)
         lake = Lake(initial_value)
@@ -424,21 +530,23 @@ class TestUpdateORM:
 
         # Check what was inserted in DB
         assert isinstance(lake.geom, WKBElement)
-        wkt = session.execute(lake.geom.ST_AsText()).scalar()
+        wkt_query = lake.geom.ST_AsText()
+        wkt = session.execute(wkt_query).scalar()
         assert format_wkt(wkt) == "LINESTRING(0 0,1 1)"
         srid = session.execute(lake.geom.ST_SRID()).scalar()
         assert srid == 4326
 
-        # Set geometry to None
-        lake.geom = None
+        if dialect_name != "mysql":
+            # Set geometry to None
+            lake.geom = None
 
-        # Update in DB
-        session.flush()
+            # Update in DB
+            session.flush()
 
-        # Check what was updated in DB
-        assert lake.geom is None
-        cols = [Lake.id, Lake.geom]
-        assert session.execute(select(cols)).fetchall() == [(1, None)]
+            # Check what was updated in DB
+            assert lake.geom is None
+            cols = [Lake.id, Lake.geom]
+            assert session.execute(select(cols)).fetchall() == [(1, None)]
 
         # Reset geometry to initial value
         lake.geom = initial_value
@@ -456,7 +564,7 @@ class TestUpdateORM:
         session.refresh(lake)
         assert to_shape(lake.geom) == to_shape(initial_value)
 
-    def test_other_type_fail(self, session, Lake, setup_tables):
+    def test_other_type_fail(self, session, Lake, setup_tables, dialect_name):
         shape = LineString([[0, 0], [1, 1]])
         lake = Lake(from_shape(shape, srid=4326))
         session.add(lake)
@@ -468,17 +576,22 @@ class TestUpdateORM:
         lake.geom = 1
 
         # Update in DB
-        if session.bind.dialect.name != "sqlite":
+        if dialect_name == "postgresql":
             with pytest.raises(ProgrammingError):
                 # Call __eq__() operator of _SpatialElement with 'other' argument equal to 1
                 # so the lake instance is detected as different and is thus updated but with
                 # an invalid geometry.
                 session.flush()
-        else:
+        elif dialect_name in ["sqlite", "geopackage"]:
             # SQLite silently set the geom attribute to NULL
             session.flush()
             session.refresh(lake)
             assert lake.geom is None
+        elif dialect_name == "mysql":
+            with pytest.raises(OperationalError):
+                session.flush()
+        else:
+            raise ValueError(f"Unexpected dialect: {dialect_name}")
 
 
 class TestCallFunction:
@@ -498,10 +611,10 @@ class TestCallFunction:
         session.expire(p)
         return p.id
 
-    def test_ST_GeometryType(self, session, Lake, setup_one_lake):
+    def test_ST_GeometryType(self, session, Lake, setup_one_lake, dialect_name):
         lake_id = setup_one_lake
 
-        if session.bind.dialect.name == "postgresql":
+        if dialect_name == "postgresql":
             expected_geometry_type = "ST_LineString"
         else:
             expected_geometry_type = "LINESTRING"
@@ -521,6 +634,7 @@ class TestCallFunction:
         assert isinstance(r4, Lake)
         assert r4.id == lake_id
 
+    @test_only_with_dialects("postgresql", "sqlite")
     def test_ST_Buffer(self, session, Lake, setup_one_lake):
         lake_id = setup_one_lake
 
@@ -546,6 +660,7 @@ class TestCallFunction:
         assert isinstance(r4, Lake)
         assert r4.id == lake_id
 
+    @test_only_with_dialects("postgresql", "sqlite")
     def test_ST_AsGeoJson(self, session, Lake, setup_one_lake):
         lake_id = setup_one_lake
         lake = session.query(Lake).get(lake_id)
@@ -578,48 +693,48 @@ class TestCallFunction:
         assert loads(r1_func) == {"type": "LineString", "coordinates": [[0, 0], [1, 1]]}
 
     @skip_case_insensitivity()
+    @test_only_with_dialects("postgresql", "mysql", "sqlite-spatialite3", "sqlite-spatialite4")
     def test_comparator_case_insensitivity(self, session, Lake, setup_one_lake):
         lake_id = setup_one_lake
 
-        s = select([func.ST_Buffer(Lake.__table__.c.geom, 2)])
+        s = select([func.ST_Transform(Lake.__table__.c.geom, 2154)])
         r1 = session.execute(s).scalar()
         assert isinstance(r1, WKBElement)
 
         lake = session.query(Lake).get(lake_id)
-        assert isinstance(lake.geom, WKBElement)
 
-        r2 = session.execute(lake.geom.ST_Buffer(2)).scalar()
+        r2 = session.execute(lake.geom.ST_Transform(2154)).scalar()
         assert isinstance(r2, WKBElement)
 
-        r3 = session.execute(lake.geom.st_buffer(2)).scalar()
+        r3 = session.execute(lake.geom.st_transform(2154)).scalar()
         assert isinstance(r3, WKBElement)
 
-        r4 = session.execute(lake.geom.St_BuFfEr(2)).scalar()
+        r4 = session.execute(lake.geom.St_TrAnSfOrM(2154)).scalar()
         assert isinstance(r4, WKBElement)
 
-        r5 = session.query(Lake.geom.ST_Buffer(2)).scalar()
+        r5 = session.query(Lake.geom.ST_Transform(2154)).scalar()
         assert isinstance(r5, WKBElement)
 
-        r6 = session.query(Lake.geom.st_buffer(2)).scalar()
+        r6 = session.query(Lake.geom.st_transform(2154)).scalar()
         assert isinstance(r6, WKBElement)
 
-        r7 = session.query(Lake.geom.St_BuFfEr(2)).scalar()
+        r7 = session.query(Lake.geom.St_TrAnSfOrM(2154)).scalar()
         assert isinstance(r7, WKBElement)
 
         assert r1.data == r2.data == r3.data == r4.data == r5.data == r6.data == r7.data
 
-    def test_unknown_function_column(self, session, Lake, setup_one_lake):
+    def test_unknown_function_column(self, session, Lake, setup_one_lake, dialect_name):
         s = select([func.ST_UnknownFunction(Lake.__table__.c.geom, 2)])
-        exc = ProgrammingError if session.bind.dialect.name == "postgresql" else OperationalError
+        exc = ProgrammingError if dialect_name == "postgresql" else OperationalError
         with pytest.raises(exc, match="ST_UnknownFunction"):
             session.execute(s)
 
-    def test_unknown_function_element(self, session, Lake, setup_one_lake):
+    def test_unknown_function_element(self, session, Lake, setup_one_lake, dialect_name):
         lake_id = setup_one_lake
         lake = session.query(Lake).get(lake_id)
 
         s = select([func.ST_UnknownFunction(lake.geom, 2)])
-        exc = ProgrammingError if session.bind.dialect.name == "postgresql" else OperationalError
+        exc = ProgrammingError if dialect_name == "postgresql" else OperationalError
         with pytest.raises(exc):
             # TODO: here the query fails because of a
             # "(psycopg2.ProgrammingError) can't adapt type 'WKBElement'"
@@ -635,9 +750,11 @@ class TestCallFunction:
 
 
 class TestShapely:
-    def test_to_shape(self, session, Lake, setup_tables):
-        if session.bind.dialect.name == "sqlite":
+    def test_to_shape(self, session, Lake, setup_tables, dialect_name):
+        if dialect_name in ["sqlite", "geopackage"]:
             data_type = str
+        elif dialect_name == "mysql":
+            data_type = bytes
         else:
             data_type = memoryview
 
@@ -675,13 +792,14 @@ class TestContraint:
             id = Column(Integer, primary_key=True)
             a_str = Column(String, nullable=True)
             checked_str = Column(String, nullable=True)
-            geom = Column(Geometry(geometry_type="LINESTRING", srid=4326, management=False))
+            geom = Column(Geometry(geometry_type="LINESTRING", srid=4326))
 
             def __init__(self, geom):
                 self.geom = geom
 
         return ConstrainedLake
 
+    @test_only_with_dialects("postgresql", "sqlite-spatialite3", "sqlite-spatialite4")
     def test_insert(self, conn, ConstrainedLake, setup_tables):
         # Insert geometries
         conn.execute(
@@ -713,7 +831,8 @@ class TestReflection:
         reflection_tables_metadata.drop_all(conn, checkfirst=True)
         reflection_tables_metadata.create_all(conn)
 
-    def test_reflection(self, conn, setup_reflection_tables):
+    @test_only_with_dialects("postgresql", "sqlite")
+    def test_reflection(self, conn, setup_reflection_tables, dialect_name):
         skip_pg12_sa1217(conn)
         t = Table(
             "lake",
@@ -721,7 +840,7 @@ class TestReflection:
             autoload_with=conn,
         )
 
-        if conn.dialect.name == "postgresql":
+        if dialect_name == "postgresql":
             # Check index query with explicit schema
             t_with_schema = Table("lake", MetaData(), autoload_with=conn, schema="gis")
             assert sorted([col.name for col in t.columns]) == sorted(
@@ -731,7 +850,7 @@ class TestReflection:
                 [idx.name for idx in t_with_schema.indexes]
             )
 
-        if get_postgis_version(conn).startswith("1."):
+        if get_postgis_major_version(conn) == 1:
             type_ = t.c.geom.type
             assert isinstance(type_, Geometry)
             assert type_.geometry_type == "GEOMETRY"
@@ -743,29 +862,30 @@ class TestReflection:
             assert type_.srid == 4326
             assert type_.dimension == 2
 
-            type_ = t.c.geom_no_idx.type
-            assert isinstance(type_, Geometry)
-            assert type_.geometry_type == "LINESTRING"
-            assert type_.srid == 4326
-            assert type_.dimension == 2
+            if dialect_name != "geopackage":
+                type_ = t.c.geom_no_idx.type
+                assert isinstance(type_, Geometry)
+                assert type_.geometry_type == "LINESTRING"
+                assert type_.srid == 4326
+                assert type_.dimension == 2
 
-            type_ = t.c.geom_z.type
-            assert isinstance(type_, Geometry)
-            assert type_.geometry_type == "LINESTRINGZ"
-            assert type_.srid == 4326
-            assert type_.dimension == 3
+                type_ = t.c.geom_z.type
+                assert isinstance(type_, Geometry)
+                assert type_.geometry_type == "LINESTRINGZ"
+                assert type_.srid == 4326
+                assert type_.dimension == 3
 
-            type_ = t.c.geom_m.type
-            assert isinstance(type_, Geometry)
-            assert type_.geometry_type == "LINESTRINGM"
-            assert type_.srid == 4326
-            assert type_.dimension == 3
+                type_ = t.c.geom_m.type
+                assert isinstance(type_, Geometry)
+                assert type_.geometry_type == "LINESTRINGM"
+                assert type_.srid == 4326
+                assert type_.dimension == 3
 
-            type_ = t.c.geom_zm.type
-            assert isinstance(type_, Geometry)
-            assert type_.geometry_type == "LINESTRINGZM"
-            assert type_.srid == 4326
-            assert type_.dimension == 4
+                type_ = t.c.geom_zm.type
+                assert isinstance(type_, Geometry)
+                assert type_.geometry_type == "LINESTRINGZM"
+                assert type_.srid == 4326
+                assert type_.dimension == 4
 
         # Drop the table
         t.drop(bind=conn)
@@ -773,9 +893,11 @@ class TestReflection:
         # Check the indexes
         check_indexes(
             conn,
+            dialect_name,
             {
                 "postgresql": [],
                 "sqlite": [],
+                "geopackage": [],
             },
             table_name=t.name,
         )
@@ -784,25 +906,32 @@ class TestReflection:
         t.create(bind=conn)
 
         # Check the indexes
-        col_attributes = _get_spatialite_attrs(conn, t.name, "geom")
-        if isinstance(col_attributes[2], int):
-            sqlite_indexes = [
-                ("lake", "geom", 2, 2, 4326, 1),
-                ("lake", "geom_m", 2002, 3, 4326, 1),
-                ("lake", "geom_no_idx", 2, 2, 4326, 0),
-                ("lake", "geom_z", 1002, 3, 4326, 1),
-                ("lake", "geom_zm", 3002, 4, 4326, 1),
-            ]
+        if dialect_name in ["sqlite", "geopackage"]:
+            if dialect_name == "geopackage":
+                col_attributes = _get_spatialite_attrs_gpkg(conn, t.name, "geom")
+            else:
+                col_attributes = _get_spatialite_attrs_sqlite(conn, t.name, "geom")
+            if isinstance(col_attributes[0], int):
+                sqlite_indexes = [
+                    ("lake", "geom", 2, 2, 4326, 1),
+                    ("lake", "geom_m", 2002, 3, 4326, 1),
+                    ("lake", "geom_no_idx", 2, 2, 4326, 0),
+                    ("lake", "geom_z", 1002, 3, 4326, 1),
+                    ("lake", "geom_zm", 3002, 4, 4326, 1),
+                ]
+            else:
+                sqlite_indexes = [
+                    ("lake", "geom", "LINESTRING", "XY", 4326, 1),
+                    ("lake", "geom_m", "LINESTRING", "XYM", 4326, 1),
+                    ("lake", "geom_no_idx", "LINESTRING", "XY", 4326, 0),
+                    ("lake", "geom_z", "LINESTRING", "XYZ", 4326, 1),
+                    ("lake", "geom_zm", "LINESTRING", "XYZM", 4326, 1),
+                ]
         else:
-            sqlite_indexes = [
-                ("lake", "geom", "LINESTRING", "XY", 4326, 1),
-                ("lake", "geom_m", "LINESTRING", "XYM", 4326, 1),
-                ("lake", "geom_no_idx", "LINESTRING", "XY", 4326, 0),
-                ("lake", "geom_z", "LINESTRING", "XYZ", 4326, 1),
-                ("lake", "geom_zm", "LINESTRING", "XYZM", 4326, 1),
-            ]
+            sqlite_indexes = []
         check_indexes(
             conn,
+            dialect_name,
             {
                 "postgresql": [
                     (
@@ -827,10 +956,12 @@ class TestReflection:
                     ),
                 ],
                 "sqlite": sqlite_indexes,
+                "geopackage": [("lake", "geom", "gpkg_rtree_index")],
             },
             table_name=t.name,
         )
 
+    @test_only_with_dialects("postgresql", "sqlite")
     def test_raster_reflection(self, conn, Ocean, setup_tables):
         skip_pg12_sa1217(conn)
         skip_postgis1(conn)
@@ -843,9 +974,12 @@ class TestReflection:
         assert isinstance(type_, Raster)
 
     @test_only_with_dialects("sqlite")
-    def test_sqlite_reflection_with_discarded_col(self, conn, Lake, setup_tables):
+    def test_sqlite_reflection_with_discarded_col(self, conn, Lake, setup_tables, dialect_name):
         """Test that a discarded geometry column is not properly reflected with SQLite."""
-        conn.execute(text("""DELETE FROM "geometry_columns" WHERE f_table_name = 'lake';"""))
+        if dialect_name == "geopackage":
+            conn.execute(text("""DELETE FROM "gpkg_geometry_columns" WHERE table_name = 'lake';"""))
+        else:
+            conn.execute(text("""DELETE FROM "geometry_columns" WHERE f_table_name = 'lake';"""))
         t = Table(
             "lake",
             MetaData(),
@@ -856,7 +990,6 @@ class TestReflection:
         assert t.c.geom.type.geometry_type == "GEOMETRY"
         assert t.c.geom.type.dimension == 2
         assert t.c.geom.type.extended
-        assert not t.c.geom.type.management
         assert t.c.geom.type.nullable
         assert t.c.geom.type.spatial_index
         assert t.c.geom.type.srid == -1
@@ -867,6 +1000,7 @@ class TestReflection:
         yield Ocean
         conn.execute(text("DROP VIEW test_view;"))
 
+    @test_only_with_dialects("postgresql", "sqlite")
     def test_view_reflection(self, conn, Ocean, setup_tables, ocean_view):
         """Test reflection of a view.
 
